
#ifndef EBOARD_STL_H
#define EBOARD_STL_H 1

#include "config.h"

#include <list>
#include <vector>
#include <deque>
#include <stack>
#include <algorithm>
#include <string>

#include <iostream>
#include <fstream>

using namespace std;

/*
using std::list;
using std::vector;
using std::stack;
using std::deque;
using std::string;

using std::ostream;
using std::istream;
using std::ofstream;
using std::ifstream;
*/

#endif
