#ifndef CONFIG_H
#define CONFIG_H 1

#define HAVE_STDIO_H 1
#define HAVE_STDLIB_H 1
#define HAVE_STRING_H 1
#define HAVE_UNISTD_H 1
#define HAVE_TIME_H 1
#define HAVE_STDARG_H 1
#define HAVE_CTYPE_H 1
#define HAVE_ERRNO_H 1
#define HAVE_FCNTL_H 1
#define HAVE_DIRENT_H 1
#define HAVE_SYS_STAT_H 1
#define HAVE_SYS_TYPES_H 1
#define HAVE_SYS_WAIT_H 1
#define HAVE_SIGNAL_H 1
#define HAVE_MATH_H 1
#define HAVE_SYS_TIME_H 1
#define HAVE_SYS_IOCTL_H 1
#define HAVE_SYS_SOCKET_H 1
#define HAVE_NETDB_H 1
#define HAVE_NETINET_IN_H 1
#define HAVE_ARPA_INET_H 1
#define HAVE_IOSTREAM 1
#define HAVE_DEQUE 1
#define HAVE_LIST 1
#define HAVE_VECTOR 1
#define HAVE_STACK 1
#define HAVE_STRING 1
#define HAVE_STRINGS_H 1
#define HAVE_SYS_SOUNDCARD_H 1
#undef HAVE_SYS_AUDIOIO_H
#define HAVE_IPPROTO_TCP_ON_IN_H 1
#undef HAVE_TCP_NODELAY_ON_IN_H
#undef HAVE_SOL_TCP_ON_IN_H
#undef HAVE_IPPROTO_TCP_ON_TCP_H
#define HAVE_TCP_NODELAY_ON_TCP_H 1
#define HAVE_SOL_TCP_ON_TCP_H 1
#define NEED_TCP_H 1
#define USE_SOCK_OPTS 1
#define USE_IPPROTO_TCP 1
#define HAVE_GTK_GTK_H 1
#define HAVE_GDK_GDKKEYSYMS_H 1
#define HAVE_GDK_IMLIB_H 1
#define HAVE_X11_XLIB_H 1
#define DATADIR "/usr/local/share"
#define EBOARD_PREFIX "/usr/local"
#define VERSION "0.9.5"
#define ENABLE_NLS 1

#endif
